@@ -29,7 +29,7 @@ our @EXPORT = qw(
 our @ISA = qw(Exporter);
 
 # Set the package version. 
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
 # Load the Perl modules.
 use POSIX qw(ttyname);
@@ -56,18 +56,22 @@ my %termhash = (
     "alacritty"              => "Alacritty",               # successful tested 
     "altyo"                  => "AltYo",                   # known, but not tested 
     "aterm"                  => "aterm",                   # successful tested
+    "black-screen"           => "Black Screen",            # know, but not tested
     "cool-retro-term"        => "Cool Retro Term",         # successful tested
     "deepin-terminal"        => "Deepin Terminal",         # successful tested
     "domterm"                => "DomTerm",                 # know, but not tested
     "Eterm"                  => "Eterm",                   # successful tested
+    "evilvte"                => "EvilVTE",                 # know, but not tested
     "extraterm"              => "Extraterm",               # know, but not tested 
     "fbpad"                  => "Fbpad",                   # know, but not tested
     "fbterm"                 => "Fbterm",                  # know, but not tested
     "finalterm"              => "Final Term",              # know, but not tested  
+    "fish"                   => "FISH",                    # know, but not tested
     "flexterm"               => "FlexTerm",                # know, but not tested
     "foot"                   => "foot",                    # know, but not tested
     "gnome-terminal-server"  => "Gnome Terminal",          # successful tested
     "guake"                  => "Guake Terminal",          # successful tested
+    "havoc"                  => "Havoc",                   # know, but not tested
     "hyper"                  => "Hyper",                   # successful tested
     "kitty"                  => "kitty",                   # successful tested
     "konsole"                => "Konsole",                 # successful tested
@@ -75,7 +79,7 @@ my %termhash = (
     "lxterminal"             => "LXTerminal",              # successful tested
     "mate-terminal"          => "MATE-Terminal",           # successful tested
     "mlterm"                 => "mlterm",                  # successful tested
-    "mlterm-tiny"            => "mlterm-tiny",             # know, but not tested
+    "mlterm-tiny"            => "mlterm-tiny",             # successful tested
     "pantheon-terminal"      => "Pantheon Terminal",       # know, but not tested 
     "pterm"                  => "pterm",                   # successful tested
     "qterminal"              => "QTerminal",               # successful tested
@@ -95,35 +99,44 @@ my %termhash = (
     "termite"                => "Termite",                 # know, but not tested
     "tilda"                  => "Tilda",                   # successful tested
     "tilix"                  => "Tilix",                   # successful tested
+    "tinyterm"               => "TinyTERM",                # know, but not tested
     "tmux"                   => "tmux",                    # successful tested
+    "tym"                    => "tym",                     # know, but not tested 
     "urxvt"                  => "urxvt",                   # successful tested
     "uxterm"                 => "UXTerm",                  # successful tested
+    "wazaterm"               => "Wazaterm",                # know, but not tested
+    "wterm"                  => "Wterm",                   # know, but not tested
     "wezterm"                => "WezTerm",                 # Test failed
     "xfce4-terminal"         => "Xfce4-Terminal-Emulator", # successful tested
+    "xiki"                   => "Xiki",                    # know, but not tested
     "xiterm+thai"            => "xiterm+thai",             # successful tested
     "xterm"                  => "XTerm",                   # successful tested
     "xvt"                    => "Xvt",                     # know, but not tested
     "yakuake"                => "Yakuake"                  # successful tested
 );
 
+# Set some variables.
+our $FN_PASSWD = "/etc/passwd";
+our $FN_SHELLS = "/etc/shells";
+
 #------------------------------------------------------------------------------#
 # Subroutine trim                                                              #
 #                                                                              #
 # Description:                                                                 #
 # The subroutine removes white spaces from both ends of a string. This is done #
 # by a logical or operation and using \s from regular expressions. Anchors are #
-# begin of string and end of of string.                                        #
+# begin ^ of string and end $ of string.                                       #
 #                                                                              #
-# @argument: $_[0] => $str  String to trim  (scalar)                           #
-# @return:   $str           Trimmed string  (scalar)                           #
+# @argument: $_[0] => $string  String to trim  (scalar)                        #
+# @return:   $string           Trimmed string  (scalar)                        #
 #------------------------------------------------------------------------------# 
 sub trim {
     # Assign the function argument to the local string variable $str.  
-    my $str = $_[0];
+    my $string = ((defined $_[0] && $_[0] ne "") ? $_[0] : "");
     # Trim the string from the left side and the right side.
-    $str =~ s/^\s+|\s+$//g;
+    $string =~ s/^\s+|\s+$//g;
     # Return the trimmed string.
-    return $str;    
+    return $string;    
 };
 
 #------------------------------------------------------------------------------# 
@@ -131,19 +144,28 @@ sub trim {
 #                                                                              #
 # Description:                                                                 #
 # The subroutine adds the square brackets [ and ] to the searchstring for      #
-# use with grep. E.g. shell becomes [s]hell.                                   # 
+# use with grep. E.g. shell becomes [s]hell. Therefor the string must have     #
+# a minumum length of 1.                                                       # 
 #                                                                              #
-# @argument: $_[0]  Searchstring                       (scalar)                #
-# @return:   $str   Searchstring with square brackets  (scalar)                #
+# @argument: $_[0] => $str  Searchstring                       (scalar)        #
+# @return:   $str           Searchstring with square brackets  (scalar)        #
 #------------------------------------------------------------------------------# 
 sub search_brackets {
-    # Assign the subroutine argument to the local variable.  
-    my $str = $_[0];
+    # Initialise the local variable $searchstring.
+    my $searchstring = "";
+    # Check if $searchstring is defined and is not empty.
+    if ((defined $_[0]) && (length($_[0]) > 0)) {
+        # Assign the subroutine argument to the local variable.  
+        $searchstring = $_[0];
+    } else {
+        # Return string of length 0. 
+        return "";
+    }
     # Add square brackets [ and ] to the given string.
-    substr($str, 0, 0) = '[';
-    substr($str, 2, 0) = ']';
+    substr($searchstring, 0, 0) = '[';
+    substr($searchstring, 2, 0) = ']';
     # Return the modified string.
-    return $str;
+    return $searchstring;
 };
 
 #------------------------------------------------------------------------------# 
@@ -428,6 +450,30 @@ sub terminal_identifier {
     return $terminal_command;
 };
 
+#------------------------------------------------------------------------------# 
+# Subroutine check_prereq                                                      # 
+#                                                                              #
+# Check package prerequisites.                                                 #
+#------------------------------------------------------------------------------# 
+sub check_prereq {
+    # Check if Linux command which exists.
+    if (! -x "/bin/which" || ! -x "/usr/bin/which") {
+        die "Linux command 'which' does not exist.\n";
+    };
+    # Check if grep, users and ps exists.
+    die unless `which grep`;
+    die unless `which users`;
+    die unless `which ps`;
+    # Check if passwd exists.
+    if (! -e $FN_PASSWD) {
+        die "$!, $FN_PASSWD not exists";
+    };
+    # Check if shells exists.
+    if (! -e $FN_SHELLS) {
+        die "$!, $FN_SHELLS not exists";
+    };
+};
+
 #------------------------------------------------------------------------------# 
 # Subroutine whichterminalami                                                  # 
 #                                                                              #
@@ -447,6 +493,8 @@ sub whichterminalami {
     my $terminal_ftn = "";
     my $terminal_name = "";
     my $terminal_path = "";
+    # Check prerequisites.
+    check_prereq();
     # Identify the terminal process command.
     my $terminal_command = terminal_identifier();
     # If terminal command is empty it is a unknown terminal.
@@ -571,49 +619,23 @@ or like this
 =head1 OUTPUT CONTROL
 
 The former introduced string arguments C<PROC>, C<PATH> and C<FTN> in square
-brackets  
+brackets in the method call are optional.
 
   ["PROC"|"PATH"|"FTN"]
 
-used in the method call are optional. The valid method arguments are separated
-by a logical or. Without such a given subroutine argument, the process name of
-the terminal is printed to the screen. These three arguments are controlling
-the format of the output of the identified terminal.
+Valid method arguments are separated by a logical or. Without such a given
+subroutine argument, the process name of the terminal is printed to the screen.
+These arguments are controlling the format of the output of the identified
+terminal.
 
 In addition to the method arguments introduced previously, there is a global
-package variable
+package variable which can be used to control the format of the output of the
+identified terminal.
 
   $OutputFormat
 
-which can be used to control the format of the output of the identified
-terminal. If the global package variable is set, the method arguments are
-ignored if existing in the method call.
-
-=head1 DESCRIPTION
-
-The main objective of this package is to provide a method which is capable of
-identifying the terminal emulator a logged-in user is actual using. 
-
-According to the current state of development, it can also be recognised whether
-it is the System Console or a remotely used System Console. 
-
-The logged-in user is related to a valid login shell. Knowing this, the
-logged-in user as well as the login shell of the logged-in user are determined.
-Next the terminal path to the pseudo terminal slave (pts) is identified. Based
-on this informations the related process of the logged-in user, the login shell
-and the terminal path is determined. The evaluation of the PID of the process of
-the running Perl script results in the PPID. The command related to this PPID
-is the name of the terminal emulator in use. Not only a terminal emulator can be
-identified by this package, it is also able to detect if a system console is
-being used by a logged-in user. The package also works if different terminal
-emulators are used and when they were spawned from on initial started terminal
-emulator.
-
-Changing a user or using sudo are currently no longer a problem for the detection
-of a terminal emulator.
-
-The terminal emulator in use by the logged-in user can be identified by the
-main command C<whichterminalami()> and the other defined aliases.
+If the global package variable is set, the method arguments are ignored if
+existing in the method call.
 
 =head1 OUTPUT FORMAT
 
@@ -630,6 +652,34 @@ arguments C<"PROC">, C<"PATH"> and C<"FTN">.
 
 =back
 
+=head1 DESCRIPTION
+
+The main objective of this package is to provide a method which is capable of
+identifying the terminal emulator a logged-in user is actual using. 
+
+In addition to the terminal emulator, the system console and a remote console
+are also recognised.
+
+The logged-in user is related to a valid login shell directly. The login shell
+of the logged-in user as well as the logged-in user is determined. Next the
+terminal path to the pseudo terminal slave (pts) is identified.
+
+Based on the previously informations the related process of the logged-in user,
+the login shell and the terminal path is determined. The evaluation of the PID
+of the process of the current running Perl script results in the PPID. The
+command related to this PPID is the name of the terminal emulator in use.
+
+The package works together with different terminal emulators. When terminal
+emulators are spawned from an initial invoked terminal emulator, each terminal
+emulator is correctly recognised.
+
+If the logged-in user changes during the session, this is recognised. Also 
+using the sudo command does not affect the recognition of the terminal
+emulator. 
+
+The terminal emulator in use by the logged-in user can be identified by the
+main command C<whichterminalami()> or the other defined aliases.
+
 =head1 EXAMPLES
 
 =head2 Example 1
@@ -699,7 +749,7 @@ operating systems in general until something else was shown.
 
 =head1 FUNCTIONALITY REQUIREMENT
 
-Following Linux commands should be available for functionality:
+The following Linux commands must be available for package functionality:
 
 =over 4 
 
@@ -711,9 +761,15 @@ Following Linux commands should be available for functionality:
 
 =item * grep
 
-=item * shells
+=back
 
-=item * passwd
+The subsequent system files must be exist for package functionality:
+
+=over 4 
+
+=item * /etc/shells
+
+=item * /etc/passwd
 
 =back
 
@@ -759,6 +815,8 @@ Terminal emulators tested so far with the package:
 
 =item * mlterm
 
+=item * mlterm-tiny
+
 =item * pterm
 
 =item * QTerminal
